#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static unsigned char checksig(unsigned char *data)
{
	if ((data[0] == 0x55) && (data[1] == 0xAA))
	{
		return 1;
	}

	return 0;
}

static int getromlen(unsigned char *data)
{
	return (data[2] * 512);
}

static int getindoff(unsigned char *data)
{
	return (data[0x18] + data[0x19] * 256 + 0x15);
}

static unsigned char dochksum(unsigned char *data, int romlen)
{
	int chkdone = 0;
	unsigned char chksum = 0;

	while (chkdone < (romlen - 1))
	{
		chksum = chksum + data[chkdone];

		++chkdone;
	}

	return (0x100 - chksum);
}

static void doindfix(unsigned char *data, unsigned char lastrom)
{
	int indoff = getindoff(data);

	if (lastrom)
	{
		data[indoff] |= 0x80;
	} else {
		data[indoff] &= 0x7F;
	}
}

int main(int argc, char **argv)
{
	FILE *f = NULL;
	unsigned char *data = NULL;
	unsigned char *nextdata = NULL;
	int filelen = 0;
	int romlen = 0;
	int romdone = 0;
	unsigned char lastrom = 0;

	if (argc != 3)
	{
		return 1;
	}

	f = fopen(argv[1], "rb");
	fseek(f, 0, SEEK_END);
	filelen = ftell(f);
	fseek(f, 0, SEEK_SET);

	printf("ROM File length: %d\n", filelen);

	if (filelen <= 0)
	{
		return -1;
	}

	data = (unsigned char *)malloc(filelen);

	fread(data, filelen, 1, f);

	fclose(f);

	nextdata = data;

	while(romdone < filelen)
	{
		if (!checksig(nextdata))
		{
			printf("ERROR: Bad ROM file entry signature (%.2X %.2X)\n", nextdata[0], nextdata[1]);

			return -1;
		}

		romlen = getromlen(nextdata);

		romdone += romlen;

		if (romdone == filelen)
		{
			lastrom = 1;
		} else if (checksig(nextdata + romdone)) {
			lastrom = 0;
		} else {
			lastrom = 1;
		}

		doindfix(nextdata, lastrom);

		nextdata[romlen-1] = dochksum(nextdata, romlen-1);

		printf("ROM Offset: %d, Length: %d, Last ROM: %d, Checksum: 0x%.2X At Offset 0x%.2X\n", romdone-romlen, romlen, lastrom, dochksum(nextdata, romlen-1), romdone-1);

		nextdata += romlen;
	}

	f = fopen(argv[2], "wb");

	fwrite(data, filelen, 1, f);

	fclose(f);

	free(data);

	return 0;
}
